@@ -35,30 +35,44 @@
 
 #include <xfconf/xfconf.h>
 #include <libxfce4ui/libxfce4ui.h>
+#include <libnotify/notify.h>
 
 #include "xfce4-notifyd-config.ui.h"
 
-/* unfortunately, currently we have to kill the daemon to
- * change themes.  this is only annoying because existing notifications
- * will get killed */
 static void
-xfce4_notifyd_config_kill_daemon(void)
+xfce_notifyd_config_show_notification_callback(NotifyNotification *notification,
+                                               const char         *action,
+                                               gpointer            unused)
 {
-    DBusGConnection *dbus_conn;
-    DBusGProxy *proxy;
+  /* Don't do anything, we just have a button to show its style */
+}
 
-    dbus_conn = dbus_g_bus_get(DBUS_BUS_SESSION, NULL);
-    if(!dbus_conn)
-        return;
+static void
+xfce_notifyd_config_show_notification_preview(GtkWindow *parent_window)
+{
+    NotifyNotification *notification;
+    GError             *error = NULL;
 
-    proxy = dbus_g_proxy_new_for_name(dbus_conn,
-                                      "org.freedesktop.Notifications",
-                                      "/org/freedesktop/Notifications",
-                                      "org.xfce.Notifyd");
-    dbus_g_proxy_call_no_reply(proxy, "Quit", G_TYPE_INVALID);
+    notification =
+        notify_notification_new(_("Notification Preview"),
+                                _("This is how notifications will look like"),
+                                "xfce4-notifyd");
 
-    g_object_unref(G_OBJECT(proxy));
-    dbus_g_connection_unref(dbus_conn);
+    notify_notification_add_action(notification,
+                                   "button",
+                                   _("Button"),
+                                   (NotifyActionCallback) xfce_notifyd_config_show_notification_callback,
+                                   NULL,
+                                   NULL);
+
+    if (!notify_notification_show(notification, &error)) {
+        xfce_dialog_show_error(parent_window, error,
+                               _("Notification preview failed"));
+
+        g_error_free(error);
+    }
+
+    g_object_unref(notification);
 }
 
 static gchar *
@@ -70,17 +84,19 @@ xfce4_notifyd_slider_format_value(GtkScale *slider,
 }
 
 static void
-xfce4_notifyd_config_theme_treeview_changed(GtkTreeSelection *sel,
-                                            gpointer user_data)
+xfce4_notifyd_config_theme_combo_changed(GtkComboBox *theme_combo,
+                                         gpointer user_data)
 {
     XfconfChannel *channel = user_data;
-    GtkTreeModel *model = NULL;
+    GtkTreeModel *model;
     GtkTreeIter iter;
     gchar *theme = NULL;
 
-    if(!gtk_tree_selection_get_selected(sel, &model, &iter))
+    if(!gtk_combo_box_get_active_iter(theme_combo, &iter))
         return;
 
+    model = gtk_combo_box_get_model (theme_combo);
+
     gtk_tree_model_get(model, &iter, 0, &theme, -1);
     xfconf_channel_set_string(channel, "/theme", theme);
     g_free(theme);
@@ -92,17 +108,15 @@ xfce4_notifyd_config_theme_changed(XfconfChannel *channel,
                                    const GValue *value,
                                    gpointer user_data)
 {
-    GtkWidget *treeview = user_data;
+    GtkWidget *theme_combo = user_data;
     GtkListStore *ls;
-    GtkTreeSelection *sel;
     GtkTreeIter iter;
     gchar *theme;
     const gchar *new_theme;
 
     new_theme = G_VALUE_TYPE(value) ? g_value_get_string(value) : "Default";
 
-    sel = gtk_tree_view_get_selection(GTK_TREE_VIEW(treeview));
-    ls = GTK_LIST_STORE(gtk_tree_view_get_model(GTK_TREE_VIEW(treeview)));
+    ls = GTK_LIST_STORE(gtk_combo_box_get_model(GTK_COMBO_BOX(theme_combo)));
 
     for(gtk_tree_model_get_iter_first(GTK_TREE_MODEL(ls), &iter);
         gtk_list_store_iter_is_valid(ls, &iter);
@@ -110,9 +124,13 @@ xfce4_notifyd_config_theme_changed(XfconfChannel *channel,
     {
         gtk_tree_model_get(GTK_TREE_MODEL(ls), &iter, 0, &theme, -1);
         if(!strcmp(theme, new_theme)) {
-            gtk_tree_selection_select_iter(sel, &iter);
+
+            gtk_combo_box_set_active_iter(GTK_COMBO_BOX(theme_combo),
+                                          &iter);
             g_free(theme);
-            xfce4_notifyd_config_kill_daemon();
+
+            xfce_notifyd_config_show_notification_preview(GTK_WINDOW(gtk_widget_get_toplevel(theme_combo)));
+
             return;
         }
         g_free(theme);
@@ -120,7 +138,7 @@ xfce4_notifyd_config_theme_changed(XfconfChannel *channel,
 
     gtk_list_store_append(ls, &iter);
     gtk_list_store_set(ls, &iter, 0, new_theme, -1);
-    gtk_tree_selection_select_iter(sel, &iter);
+    gtk_combo_box_set_active_iter(GTK_COMBO_BOX(theme_combo), &iter);
 }
 
 static void
@@ -132,7 +150,7 @@ list_store_add_themes_in_dir(GtkListStore *ls,
 {
     GDir *dir;
     const gchar *file;
-    gchar buf[PATH_MAX];
+    gchar *filename;
 
     dir = g_dir_open(path, 0, NULL);
     if(!dir)
@@ -142,9 +160,10 @@ list_store_add_themes_in_dir(GtkListStore *ls,
         if(g_hash_table_lookup(themes_seen, file))
             continue;
 
-        g_snprintf(buf, sizeof(buf), "%s/%s/xfce-notify-4.0/gtkrc",
-                   path, file);
-        if(g_file_test(buf, G_FILE_TEST_IS_REGULAR)) {
+        filename =
+            g_build_filename(path, file, "xfce-notify-4.0", "gtkrc", NULL);
+
+        if(g_file_test(filename, G_FILE_TEST_IS_REGULAR)) {
             GtkTreeIter iter;
 
             gtk_list_store_append(ls, &iter);
@@ -155,23 +174,22 @@ list_store_add_themes_in_dir(GtkListStore *ls,
             if(!strcmp(file, current_theme))
                 memcpy(current_theme_iter, &iter, sizeof(iter));
         }
+
+        g_free(filename);
     }
 
     g_dir_close(dir);
 }
 
 static void
-xfce4_notifyd_config_setup_treeview(GtkWidget *treeview,
-                                    const gchar *current_theme)
+xfce4_notifyd_config_setup_theme_combo(GtkWidget *theme_combo,
+                                       const gchar *current_theme)
 {
     GtkListStore *ls;
     gchar *dirname, **dirnames;
     GHashTable *themes_seen;
     gint i;
-    GtkCellRenderer *render;
-    GtkTreeViewColumn *col;
     GtkTreeIter current_theme_iter;
-    GtkTreeSelection *sel;
 
     ls = gtk_list_store_new(1, G_TYPE_STRING);
     themes_seen = g_hash_table_new_full(g_str_hash, g_str_equal,
@@ -190,19 +208,11 @@ xfce4_notifyd_config_setup_treeview(GtkWidget *treeview,
 
     g_hash_table_destroy(themes_seen);
 
-    gtk_tree_view_set_model(GTK_TREE_VIEW(treeview), GTK_TREE_MODEL(ls));
+    gtk_combo_box_set_model(GTK_COMBO_BOX(theme_combo), GTK_TREE_MODEL(ls));
 
-    render = gtk_cell_renderer_text_new();
-    col = gtk_tree_view_column_new_with_attributes(_("Theme"), render,
-                                                   "text", 0, NULL);
-    gtk_tree_view_append_column(GTK_TREE_VIEW(treeview), col);
-
-    gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(ls), 0,
-                                         GTK_SORT_ASCENDING);
-
-    sel = gtk_tree_view_get_selection(GTK_TREE_VIEW(treeview));
     if(gtk_list_store_iter_is_valid(ls, &current_theme_iter))
-        gtk_tree_selection_select_iter(sel, &current_theme_iter);
+        gtk_combo_box_set_active_iter(GTK_COMBO_BOX(theme_combo),
+                                      &current_theme_iter);
 
     g_object_unref(G_OBJECT(ls));
 }
@@ -218,24 +228,15 @@ static void
 xfce_notifyd_config_preview_clicked(GtkButton *button,
                                     GtkWidget *dialog)
 {
-    GError *error = NULL;
-
-    if(!g_spawn_command_line_async(_("notify-send \"Notification Preview\""
-                                     " \"This is how notifications will look like\""),
-                                   &error)) {
-        xfce_dialog_show_error(GTK_WINDOW(dialog),
-                               error, _("Notification preview failed"));
-        g_error_free(error);
-    }
+    xfce_notifyd_config_show_notification_preview(GTK_WINDOW(dialog));
 }
 
 static GtkWidget *
 xfce4_notifyd_config_setup_dialog(GtkBuilder *builder)
 {
     XfconfChannel *channel;
-    GtkWidget *dlg, *btn, *sbtn, *slider, *treeview, *combo;
+    GtkWidget *dlg, *btn, *sbtn, *slider, *theme_combo, *position_combo;
     GtkAdjustment *adj;
-    GtkTreeSelection *sel;
     GError *error = NULL;
     gchar *current_theme;
 
@@ -276,24 +277,23 @@ xfce4_notifyd_config_setup_dialog(GtkBuilder *builder)
     xfconf_g_property_bind(channel, "/initial-opacity", G_TYPE_DOUBLE,
                            G_OBJECT(adj), "value");
 
-    treeview = GTK_WIDGET(gtk_builder_get_object(builder, "themes_treeview"));
+    theme_combo = GTK_WIDGET(gtk_builder_get_object(builder, "theme_combo"));
     current_theme = xfconf_channel_get_string(channel, "/theme", "Default");
-    xfce4_notifyd_config_setup_treeview(treeview, current_theme);
+    xfce4_notifyd_config_setup_theme_combo(theme_combo, current_theme);
     g_free(current_theme);
 
-    sel = gtk_tree_view_get_selection(GTK_TREE_VIEW(treeview));
-    g_signal_connect(G_OBJECT(sel), "changed",
-                     G_CALLBACK(xfce4_notifyd_config_theme_treeview_changed),
+    g_signal_connect(G_OBJECT(theme_combo), "changed",
+                     G_CALLBACK(xfce4_notifyd_config_theme_combo_changed),
                      channel);
     g_signal_connect(G_OBJECT(channel), "property-changed::/theme",
                      G_CALLBACK(xfce4_notifyd_config_theme_changed),
-                     treeview);
+                     theme_combo);
 
-    combo = GTK_WIDGET(gtk_builder_get_object(builder, "position_combo"));
+    position_combo = GTK_WIDGET(gtk_builder_get_object(builder, "position_combo"));
     xfconf_g_property_bind(channel, "/notify-location", G_TYPE_UINT,
-                           G_OBJECT(combo), "active");
-    if(gtk_combo_box_get_active(GTK_COMBO_BOX(combo)) == -1)
-        gtk_combo_box_set_active(GTK_COMBO_BOX(combo), GTK_CORNER_TOP_RIGHT);
+                           G_OBJECT(position_combo), "active");
+    if(gtk_combo_box_get_active(GTK_COMBO_BOX(position_combo)) == -1)
+        gtk_combo_box_set_active(GTK_COMBO_BOX(position_combo), GTK_CORNER_TOP_RIGHT);
 
     return dlg;
 }
@@ -337,6 +337,11 @@ main(int argc,
         return EXIT_SUCCESS;
     }
 
+    if (!notify_init ("Xfce4-notifyd settings")) {
+      g_error ("Failed to initialize libnotify.");
+      return EXIT_FAILURE;
+    }
+
     builder = gtk_builder_new();
     gtk_builder_add_from_string(builder, xfce4_notifyd_config_ui, xfce4_notifyd_config_ui_length, NULL);
     if(G_UNLIKELY(!builder)) {
@@ -346,7 +351,6 @@ main(int argc,
 
     settings_dialog = xfce4_notifyd_config_setup_dialog(builder);
 
-
     if(opt_socket_id) {
         GtkWidget *plug, *plug_child;
 
@@ -370,5 +374,8 @@ main(int argc,
         gtk_dialog_run(GTK_DIALOG(settings_dialog));
     }
 
+    notify_uninit();
+    xfconf_shutdown();
+
     return EXIT_SUCCESS;
 }
