@@ -1,10 +1,10 @@
 
-/* Generated data (by glib-mkenums) */
+
 
 #include "xfce-notify-enum-types.h"
-#include "xfce-notify-window.h"
+#include "xfce4-notifyd/xfce-notify-window.h"
 
-/* enumerations from "xfce4-notifyd/xfce-notify-window.h" */
+/* enumerations from "./xfce4-notifyd/xfce-notify-window.h" */
 GType
 xfce_notify_close_reason_get_type(void)
 {
@@ -22,5 +22,5 @@ xfce_notify_close_reason_get_type(void)
 	return type;
 }
 
-/* Generated data ends here */
+
 
