@@ -1,12 +1,11 @@
 #include "xfce-notify-marshal.h"
 
-
 #include	<glib-object.h>
 
 
 #ifdef G_ENABLE_DEBUG
 #define g_marshal_value_peek_boolean(v)  g_value_get_boolean (v)
-#define g_marshal_value_peek_char(v)     g_value_get_char (v)
+#define g_marshal_value_peek_char(v)     g_value_get_schar (v)
 #define g_marshal_value_peek_uchar(v)    g_value_get_uchar (v)
 #define g_marshal_value_peek_int(v)      g_value_get_int (v)
 #define g_marshal_value_peek_uint(v)     g_value_get_uint (v)
@@ -23,6 +22,7 @@
 #define g_marshal_value_peek_boxed(v)    g_value_get_boxed (v)
 #define g_marshal_value_peek_pointer(v)  g_value_get_pointer (v)
 #define g_marshal_value_peek_object(v)   g_value_get_object (v)
+#define g_marshal_value_peek_variant(v)  g_value_get_variant (v)
 #else /* !G_ENABLE_DEBUG */
 /* WARNING: This code accesses GValues directly, which is UNSUPPORTED API.
  *          Do not access GValues directly in your code. Instead, use the
@@ -46,10 +46,11 @@
 #define g_marshal_value_peek_boxed(v)    (v)->data[0].v_pointer
 #define g_marshal_value_peek_pointer(v)  (v)->data[0].v_pointer
 #define g_marshal_value_peek_object(v)   (v)->data[0].v_pointer
+#define g_marshal_value_peek_variant(v)  (v)->data[0].v_pointer
 #endif /* !G_ENABLE_DEBUG */
 
 
-/* VOID:UINT,UINT (xfce4-notifyd/xfce-notify-marshal.list:1) */
+/* VOID:UINT,UINT (./xfce4-notifyd/xfce-notify-marshal.list:1) */
 void
 xfce_notify_marshal_VOID__UINT_UINT (GClosure     *closure,
                                      GValue       *return_value G_GNUC_UNUSED,
@@ -86,7 +87,7 @@ xfce_notify_marshal_VOID__UINT_UINT (GClosure     *closure,
             data2);
 }
 
-/* VOID:UINT,STRING (xfce4-notifyd/xfce-notify-marshal.list:2) */
+/* VOID:UINT,STRING (./xfce4-notifyd/xfce-notify-marshal.list:2) */
 void
 xfce_notify_marshal_VOID__UINT_STRING (GClosure     *closure,
                                        GValue       *return_value G_GNUC_UNUSED,
